@@ -8,7 +8,7 @@
 ;;;;                Original code by Pierre R. Mai 
 ;;;; Date Started:  Feb 2002
 ;;;;
-;;;; $Id: mysql-api.lisp 9532 2004-05-30 19:16:34Z kevin $
+;;;; $Id: mysql-api.lisp 10179 2004-12-03 23:41:00Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002 by Kevin M. Rosenberg
 ;;;; and Copyright (c) 1999-2001 by Pierre R. Mai
@@ -135,7 +135,7 @@
 #+mysql-client-v4.1
 (uffi:def-struct mysql-field
     (name (* :char))
-  (org_table (* :char))
+  (org_name (* :char))
   (table (* :char))
   (org_table (* :char))
   (db (* :char))
@@ -540,8 +540,8 @@
 
 (declaim (inline mysql-escape-string))
 (uffi:def-function "mysql_escape_string"
-    ((to :cstring)
-     (from :cstring)
+    ((to (* :unsigned-char))
+     (from (* :unsigned-char))
      (length :unsigned-int))
   :module "mysql"
   :returning :unsigned-int)
