@@ -6,7 +6,7 @@
 ;;;; Purpose:  Low-level ODBC API using UFFI
 ;;;; Authors:  Kevin M. Rosenberg and Paul Meurer
 ;;;;
-;;;; $Id: odbc-api.lisp 9522 2004-05-30 08:01:53Z kevin $
+;;;; $Id: odbc-api.lisp 10077 2004-10-01 17:02:20Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2004 by Kevin M. Rosenberg
 ;;;; and Copyright (C) Paul Meurer 1999 - 2001. All rights reserved.
@@ -151,7 +151,7 @@ as possible second argument) to the desired representation of date/time/timestam
 	 (with-foreign-object (phenv 'sql-handle)
 	   (with-error-handling
 	       ()
-	     (SQLAllocEnv phenv)
+	     (SQLAllocHandle $SQL_HANDLE_ENV +null-handle-ptr+ phenv)
 	     (deref-pointer phenv 'sql-handle)))))
     (%set-attr-odbc-version henv $SQL_OV_ODBC3)
     henv))
@@ -164,9 +164,10 @@ as possible second argument) to the desired representation of date/time/timestam
 
 (defun %new-db-connection-handle (henv)
   (with-foreign-object (phdbc 'sql-handle)
+    (setf (deref-pointer phdbc sql-handle) +null-handle-ptr+)
     (with-error-handling
       (:henv henv)
-      (SQLAllocConnect henv phdbc)
+      (SQLAllocHandle $SQL_HANDLE_DBC henv phdbc)
       (deref-pointer phdbc 'sql-handle))))
 
 (defun %free-statement (hstmt option)
@@ -248,11 +249,11 @@ as possible second argument) to the desired representation of date/time/timestam
 
 (defun %new-statement-handle (hdbc)
   (let ((statement-handle
-	 (with-foreign-object (hstmt-ptr 'sql-handle)
+	 (with-foreign-object (phstmt 'sql-handle)
 	   (with-error-handling 
 	       (:hdbc hdbc)
-	     (SQLAllocStmt hdbc hstmt-ptr) 
-	     (deref-pointer hstmt-ptr 'sql-handle)))))
+	     (SQLAllocHandle $SQL_HANDLE_STMT hdbc phstmt) 
+	     (deref-pointer phstmt 'sql-handle)))))
     (if (uffi:null-pointer-p statement-handle)
 	(error 'clsql:sql-database-error :message "Received null statement handle.")
 	statement-handle)))
@@ -575,13 +576,13 @@ as possible second argument) to the desired representation of date/time/timestam
     (#.$SQL_TINYINT $SQL_C_STINYINT)
     (#.$SQL_BIT $SQL_C_BIT)))
 
-(def-type byte-pointer-type '(* :byte))
-(def-type short-pointer-type '(* :short))
-(def-type int-pointer-type '(* :int))
-(def-type long-pointer-type '(* #.$ODBC-LONG-TYPE))
-(def-type float-pointer-type '(* :float))
-(def-type double-pointer-type '(* :double))
-(def-type string-pointer-type '(* :unsigned-char))
+(def-type byte-pointer-type (* :byte))
+(def-type short-pointer-type (* :short))
+(def-type int-pointer-type (* :int))
+(def-type long-pointer-type (* #.$ODBC-LONG-TYPE))
+(def-type float-pointer-type (* :float))
+(def-type double-pointer-type (* :double))
+(def-type string-pointer-type (* :unsigned-char))
 
 (defun get-cast-byte (ptr)
   (locally (declare (type byte-pointer-type ptr))
@@ -647,7 +648,6 @@ as possible second argument) to the desired representation of date/time/timestam
 		   (#.$SQL_INTEGER (get-cast-int data-ptr))
 		   (#.$SQL_BIGINT (read-from-string
 				   (get-cast-foreign-string data-ptr)))
-		   (#.$SQL_TINYINT (get-cast-byte data-ptr))
 		   (#.$SQL_DECIMAL 
 		    (let ((*read-base* 10))
 		      (read-from-string (get-cast-foreign-string data-ptr))))
@@ -899,9 +899,9 @@ as possible second argument) to the desired representation of date/time/timestam
                (read-from-string str))
            str))))))
 
-(def-type c-timestamp-ptr-type '(* (:struct sql-c-timestamp)))
-(def-type c-time-ptr-type '(* (:struct sql-c-time)))
-(def-type c-date-ptr-type '(* (:struct sql-c-date)))
+(def-type c-timestamp-ptr-type (* (:struct sql-c-timestamp)))
+(def-type c-time-ptr-type (* (:struct sql-c-time)))
+(def-type c-date-ptr-type (* (:struct sql-c-date)))
 
 (defun timestamp-to-universal-time (ptr)
   (declare (type c-timestamp-ptr-type ptr))
