@@ -1,4 +1,4 @@
-;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Package: odbc -*-
+ ;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Package: odbc -*-
 ;;;; *************************************************************************
 ;;;; FILE IDENTIFICATION
 ;;;;
@@ -6,7 +6,7 @@
 ;;;; Purpose:  Constants for UFFI interface to ODBC
 ;;;; Authors:  Kevin M. Rosenberg and Paul Meurer
 ;;;;
-;;;; $Id: odbc-constants.lisp 9522 2004-05-30 08:01:53Z kevin $
+;;;; $Id: odbc-constants.lisp 9977 2004-09-07 20:45:43Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2004 by Kevin M. Rosenberg
 ;;;; and Copyright (C) Paul Meurer 1999 - 2001. All rights reserved.
@@ -24,6 +24,13 @@
 
 ;; (defconstant $ODBCVER	#x0210)
 
+
+;; for new SQLAllocHandle functiion
+(defconstant $SQL_HANDLE_ENV 1)
+(defconstant $SQL_HANDLE_DBC 2)
+(defconstant $SQL_HANDLE_STMT 3)
+(defconstant $SQL_HANDLE_DESC 4)
+
 ;; generally useful constants
 (defconstant $SQL_SPEC_MAJOR 2)		;; Major version of specification 
 (defconstant $SQL_SPEC_MINOR 10) 	;; Minor version of specification 
