@@ -7,7 +7,7 @@
 ;;;; Programmer:    Kevin M. Rosenberg
 ;;;; Date Started:  Feb 2002
 ;;;;
-;;;; $Id: odbc-sql.lisp 9538 2004-05-31 21:47:53Z marcusp $
+;;;; $Id: odbc-sql.lisp 9977 2004-09-07 20:45:43Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002 by Kevin M. Rosenberg
 ;;;;
@@ -52,6 +52,7 @@
 	  ;; underlying database
 	  (initialize-database-type :database-type database-type)
 	  db)
+      #+ignore
       (error () 	;; Init or Connect failed
 	(error 'sql-connection-error
 	       :database-type database-type
@@ -70,9 +71,14 @@
 	  (cond 
 	   ((or (search "postgresql" server-name :test #'char-equal)
 		(search "postgresql" dbms-name :test #'char-equal))
+	    (unless (find-package 'clsql-postgresql)
+	      (ignore-errors (asdf:operate 'asdf:load-op 'clsql-postgresql-socket)))
 	    :postgresql)
 	   ((or (search "mysql" server-name :test #'char-equal)
 		(search "mysql" dbms-name :test #'char-equal))
+	    (unless (find-package 'clsql-mysql)
+	      ;; ignore errors on platforms where the shared libraries are not available
+	      (ignore-errors (asdf:operate 'asdf:load-op 'clsql-mysql)))
 	    :mysql)
 	   ((or (search "oracle" server-name :test #'char-equal)
 		(search "oracle" dbms-name :test #'char-equal))
