@@ -7,7 +7,7 @@
 ;;;; Authors:  Kevin M. Rosenberg based on original code by Pierre R. Mai 
 ;;;; Created:  Feb 2002
 ;;;;
-;;;; $Id: postgresql-socket-api.lisp 9342 2004-05-13 10:21:23Z kevin $
+;;;; $Id: postgresql-socket-api.lisp 10155 2004-11-12 03:31:00Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2004 by Kevin M. Rosenberg
 ;;;; and Copyright (c) 1999-2001 by Pierre R. Mai
@@ -426,7 +426,7 @@ troubles."
 (defun encrypt-md5 (plaintext salt)
   (string-downcase
    (format nil "~{~2,'0X~}"
-	   (coerce (md5:md5sum-sequence (concatenate 'string plaintext salt)) 'list))))
+	   (coerce (md5sum-sequence (concatenate 'string plaintext salt)) 'list))))
 
 (defun reopen-postgresql-connection (connection)
   "Reopen the given PostgreSQL connection.  Closes any existing
@@ -458,14 +458,16 @@ connection, if it is still open."
 		    (3
 		     (send-unencrypted-password-message
 		      socket
-		      (postgresql-connection-password connection)))
+		      (postgresql-connection-password connection))
+                      (force-output socket))
 		    (4
 		     (let ((salt (make-string 2)))
 		       (read-socket-sequence salt socket)
 		       (send-encrypted-password-message
 			socket
 			(crypt-password
-			 (postgresql-connection-password connection) salt))))
+			 (postgresql-connection-password connection) salt)))
+                     (force-output socket))
 		    (5
 		     (let ((salt (make-string 4)))
 		       (read-socket-sequence salt socket)
@@ -474,7 +476,8 @@ connection, if it is still open."
 			      (pwd (encrypt-md5 pwd2 salt)))
 			 (send-encrypted-password-message
 			  socket
-			  (concatenate 'string "md5" pwd)))))
+			  (concatenate 'string "md5" pwd))))
+                     (force-output socket))
 		    (t
 		     (error 'postgresql-login-error
 			    :connection connection
