@@ -1,7 +1,7 @@
 ;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
 ;;;; *************************************************************************
 ;;;;
-;;;; $Id: metaclasses.lisp 9478 2004-05-25 10:55:10Z kevin $
+;;;; $Id: metaclasses.lisp 10190 2004-12-07 06:10:37Z kevin $
 ;;;;
 ;;;; CLSQL metaclass for standard-db-objects created in the OODDL. 
 ;;;;
@@ -52,7 +52,7 @@
     :accessor view-class-qualifier
     :initarg :qualifier
     :initform nil))
-  (:documentation "VIEW-CLASS metaclass."))
+  (:documentation "Metaclass for all CLSQL View Classes."))
 
 ;;; Lispworks 4.2 and before requires special processing of extra slot and class options
 
@@ -167,9 +167,9 @@
 
 (defun describe-db-layout (class)
   (flet ((not-db-col (col)
-           (not (member (nth 2 col)  '(nil :base :key))))
+           (not (member (nth 2 col) '(nil :base :key))))
          (frob-slot (slot)
-           (let ((type (slot-value slot 'type)))
+           (let ((type (slot-definition-type slot)))
              (if (eq type t)
                  (setq type nil))
              (list (slot-value slot 'name)
@@ -281,13 +281,13 @@ column definition in the database.")
     :initarg :db-constraints
     :initform nil
     :documentation
-    "A single constraint or list of constraints for this column")
+    "A keyword symbol representing a single SQL column constraint or list of such symbols.")
    (void-value
     :accessor view-class-slot-void-value
     :initarg :void-value
     :initform nil
     :documentation
-    "Value to store is the SQL value is NULL. Default is NIL.")
+    "Value to store if the SQL value is NULL. Default is NIL.")
    (db-info
     :accessor view-class-slot-db-info
     :initarg :db-info
@@ -423,13 +423,15 @@ which does type checking before storing a value in a slot."
       (car list)
       list))
 
-(declaim (inline delistify))
+(declaim (inline delistify-dsd))
 (defun delistify-dsd (list)
   "Some MOPs, like openmcl 0.14.2, cons attribute values in a list."
   (if (and (listp list) (null (cdr list)))
       (car list)
       list))
 
+(defvar *impl-type-attrib-name* #-clisp 'type #+clisp 'clos::$type)
+
 (defmethod compute-effective-slot-definition ((class standard-db-class)
 					      #+kmr-normal-cesd slot-name
 					      direct-slots)
@@ -442,9 +444,10 @@ which does type checking before storing a value in a slot."
 	       (null (specified-type dsd)))
       (setf (specified-type dsd)
 	(slot-definition-type dsd))
-      (setf (slot-value dsd 'type)
-	(compute-lisp-type-from-slot-specification 
-	 dsd (slot-definition-type dsd))))
+      (setf #-clisp (slot-value dsd 'type)
+	    #+clisp (slot-definition-type dsd)
+	    (compute-lisp-type-from-slot-specification 
+	     dsd (slot-definition-type dsd))))
       
     (let ((esd (call-next-method)))
       (typecase dsd
@@ -508,9 +511,9 @@ which does type checking before storing a value in a slot."
 	(t
 	 (let ((type-predicate #+openmcl (slot-value esd 'ccl::type-predicate)))
 	   #-openmcl (declare (ignore type-predicate))
-	   (change-class esd 'view-class-effective-slot-definition
-			 #+allegro :name 
-			 #+allegro (slot-definition-name dsd))
+	   #-clisp (change-class esd 'view-class-effective-slot-definition
+				 #+allegro :name 
+				 #+allegro (slot-definition-name dsd))
 	   #+openmcl (setf (slot-value esd 'ccl::type-predicate)
 			   type-predicate))
 	 
