@@ -7,7 +7,7 @@
 ;;;; Programmers:   Kevin M. Rosenberg, Marc Battyani
 ;;;; Date Started:  Apr 2002
 ;;;;
-;;;; $Id: pool.lisp 9336 2004-05-13 06:55:48Z kevin $
+;;;; $Id: pool.lisp 10104 2004-10-23 03:56:54Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002-2003 by Kevin M. Rosenberg
 ;;;;
@@ -52,7 +52,8 @@
   (with-process-lock ((conn-pool-lock pool) "Clear pool")
     (loop for conn across (all-connections pool)
 	  do (setf (conn-pool conn) nil)
-	  (disconnect :database conn))
+	  ;; disconnect may error if remote side closed connection
+	  (ignore-errors (disconnect :database conn)))
     (setf (fill-pointer (free-connections pool)) 0)
     (setf (fill-pointer (all-connections pool)) 0))
   nil)
