@@ -3,7 +3,7 @@
 ;;;; File:    test-init.lisp
 ;;;; Authors: Marcus Pearce <m.t.pearce@city.ac.uk>, Kevin Rosenberg
 ;;;; Created: 30/03/2004
-;;;; Updated: $Id: test-init.lisp 9519 2004-05-30 06:22:22Z kevin $
+;;;; Updated: $Id: test-init.lisp 10128 2004-10-31 23:59:26Z kevin $
 ;;;;
 ;;;; Initialisation utilities for running regression tests on CLSQL. 
 ;;;;
@@ -48,7 +48,7 @@
 (def-view-class employee (person)
   ((emplid
     :db-kind :key
-    :db-constraints :not-null
+    :db-constraints (:not-null :unique)
     :type integer
     :initarg :emplid)
    (groupid
@@ -164,6 +164,11 @@
 	    :db-info (:join-class address
 				  :home-key aaddressid
 				  :foreign-key addressid
+				  :retrieval :immediate))
+   (employee :db-kind :join
+	     :db-info (:join-class employee
+				  :home-key aemplid
+				  :foreign-key emplid
 				  :retrieval :immediate)))
   (:base-table "ea_join"))
 
@@ -579,6 +584,11 @@
 				:fdml/select/21 :fdml/select/32 
                                 :fdml/select/33))
 	   (push (cons test "not supported by sqlite") skip-tests))
+	  ((and (eql *test-database-type* :sqlite3)
+		(clsql-sys:in test :fddl/view/4 :fdml/select/10
+			      :fdml/select/21 :fdml/select/32 
+			      :fdml/select/33))
+	   (push (cons test "not supported by sqlite3") skip-tests))
 	  ((and (not (clsql-sys:db-type-has-bigint? db-type))
 		(clsql-sys:in test :basic/bigint/1))
 	   (push (cons test "bigint not supported") skip-tests))
@@ -593,8 +603,17 @@
 		(clsql-sys:in test :fdml/query/6 :fdml/select/31))
 	   (push (cons test "union not supported") skip-tests))
 	  ((and (eq *test-database-type* :oracle)
-		(clsql-sys:in test :fdml/query/8 :fdml/select/21))
+		(clsql-sys:in test :fdml/query/8 :fdml/select/21 
+                              :fddl/table/6))
 	   (push (cons test "syntax not supported") skip-tests))
+          ((and (eq *test-database-type* :odbc)
+		(eq *test-database-underlying-type* :postgresql)
+		(clsql-sys:in test :fddl/owner/1))
+           (push (cons test "table ownership not supported by postgresql odbc driver") skip-tests))
+	  ((and (not (member *test-database-underlying-type* 
+                             '(:postgresql :oracle)))
+		(clsql-sys:in test :fddl/owner/1))
+           (push (cons test "table ownership not supported") skip-tests))
 	  (t
 	   (push test-form test-forms)))))
       (values (nreverse test-forms) (nreverse skip-tests))))
