@@ -3,7 +3,7 @@
 ;;;; File:    test-syntax.lisp
 ;;;; Author:  Marcus Pearce <m.t.pearce@city.ac.uk>
 ;;;; Created: 30/03/2004
-;;;; Updated: $Id: test-syntax.lisp 9538 2004-05-31 21:47:53Z marcusp $
+;;;; Updated: $Id: test-syntax.lisp 9758 2004-07-12 23:45:58Z marcusp $
 ;;;; ======================================================================
 ;;;;
 ;;;; Description ==========================================================
@@ -40,6 +40,9 @@
     (clsql:sql ["SELECT FOO,BAR FROM BAZ"])
   "SELECT FOO,BAR FROM BAZ")
 
+(deftest :syntax/generic/6 
+    (clsql:sql "What's up Doc?") 
+  "'What''s up Doc?'")
 
 (deftest :syntax/ident/1
     (clsql:sql [foo])
