@@ -7,7 +7,7 @@
 ;;;; Programmers:   Kevin M. Rosenberg
 ;;;; Date Started:  Mar 2002
 ;;;;
-;;;; $Id: clsql-uffi.lisp 9482 2004-05-25 19:06:08Z kevin $
+;;;; $Id: clsql-uffi.lisp 10108 2004-10-23 15:16:15Z kevin $
 ;;;;
 ;;;; This file, part of CLSQL, is Copyright (c) 2002 by Kevin M. Rosenberg
 ;;;;
@@ -30,7 +30,7 @@
        (nreverse new-types))
     (declare (fixnum length-types length-auto-list i))
     (if (>= i length-types)
-	(push t new-types) ;; types is shorted than num-fields
+	(push t new-types) ;; types is shorter than num-fields
 	(push
 	 (case (nth i types)
 	   (:int
@@ -54,7 +54,9 @@
 	   (:int64
 	    (if (eq :int64 (nth i auto-list))
 		:int64
-		t))
+	      t))
+	   (:blob
+	    :blob)
 	   (t
 	    t))
 	 new-types))))
@@ -73,7 +75,8 @@
 
 (uffi:def-function "atol64"
     ((str (* :unsigned-char))
-     (high32 (* :int)))
+     (high32 (* :unsigned-int)))
+  :module "clsql-uffi"
   :returning :unsigned-int)
 
 (uffi:def-constant +2^32+ 4294967296)
@@ -97,7 +100,7 @@
 (defun convert-raw-field (char-ptr types index &optional length)
   (declare (optimize (speed 3) (safety 0) (space 0))
  	   (type char-ptr-def char-ptr))
-  (let ((type (if (listp types)
+  (let ((type (if (consp types)
 		  (nth index types)
 		  types)))
     (cond
@@ -107,15 +110,19 @@
        (case type
 	 (:double
 	  (atof char-ptr))
-	 ((or :int32 :int)
+	 ((:int32 :int)
 	  (atoi char-ptr))
 	 (:int64
-	  (uffi:with-foreign-object (high32-ptr :int)
+	  (uffi:with-foreign-object (high32-ptr :unsigned-int)
 	    (let ((low32 (atol64 char-ptr high32-ptr))
-		  (high32 (uffi:deref-pointer high32-ptr :int)))
+		  (high32 (uffi:deref-pointer high32-ptr :unsigned-int)))
 	      (if (zerop high32)
 		  low32
-		  (make-64-bit-integer high32 low32)))))
+		(make-64-bit-integer high32 low32)))))
+	 (:blob
+	  (if length
+	      (uffi:convert-from-foreign-usb8 char-ptr length)
+	    (error "Can't return blob since length is not specified.")))
 	 (t
           (if length
 	      (uffi:convert-from-foreign-string char-ptr :locale :none
